@@ -768,6 +768,9 @@ reattempt:
         if (is_datatype_command()) 
                 goto reattempt;
 
+	if (STRNEQ(args[0], "#") || STRNEQ(args[0], "//"))
+		return;
+
 	if (!(pc->flags & MINIMAL_MODE) &&
 	    is_gdb_command(TRUE, FAULT_ON_ERROR)) 
 		goto reattempt;
@@ -796,8 +799,23 @@ reattempt:
 struct command_table_entry *
 get_command_table_entry(char *name)
 {       
+	int i;
         struct command_table_entry *cp;
         struct extension_table *ext;
+
+	if (pc->flags2 & GDB_CMD_MODE) {
+		if (STREQ(name, "crash")) {
+			if (argcnt == 1)
+				error(FATAL, 
+				    "a crash command must follow "
+				    "the \"crash\" directive\n");
+			for (i = 1; i <= argcnt; i++)
+				args[i-1] = args[i];
+			argcnt--;
+			name = args[0];
+		} else
+			name = "gdb";
+	}
 	
 	if ((pc->flags & MINIMAL_MODE) && !minimal_functions(name)) 
 		return NULL;
@@ -1295,6 +1313,8 @@ dump_program_context(void)
 		fprintf(fp, "%sGET_OSRELEASE", others++ ? "|" : "");
 	if (pc->flags2 & REMOTE_DAEMON)
 		fprintf(fp, "%sREMOTE_DAEMON", others++ ? "|" : "");
+	if (pc->flags2 & LIVE_DUMP)
+		fprintf(fp, "%sLIVE_DUMP", others++ ? "|" : "");
 	fprintf(fp, ")\n");
 
 	fprintf(fp, "         namelist: %s\n", pc->namelist);
@@ -1486,6 +1506,8 @@ dump_program_context(void)
 		fprintf(fp, "%sMOD_READNOW", others ? "|" : "");
 	fprintf(fp, ")\n");
 	fprintf(fp, "   curcmd_private: %llx\n", pc->curcmd_private); 
+	fprintf(fp, "      cmd_cleanup: %lx\n", (ulong)pc->cmd_cleanup);
+	fprintf(fp, "  cmd_cleanup_arg: %lx\n", (ulong)pc->cmd_cleanup_arg);
 	fprintf(fp, "       sigint_cnt: %d\n", pc->sigint_cnt);
 	fprintf(fp, "        sigaction: %lx\n", (ulong)&pc->sigaction);
 	fprintf(fp, "    gdb_sigaction: %lx\n", (ulong)&pc->gdb_sigaction);
