@@ -1979,6 +1979,8 @@ dump_Elf64_Nhdr(Elf64_Off offset, int store)
 		break;
 	case NT_TASKSTRUCT:
 		netdump_print("(NT_TASKSTRUCT)\n");
+		if (STRNEQ(buf, "SNAP"))
+			pc->flags2 |= LIVE_DUMP;
 		if (store) {
 			nd->nt_taskstruct = (void *)note;
 			nd->task_struct = *((ulong *)(ptr + note->n_namesz));
